<?php
// ============================================================================
// GD-CMS: A heirarchical content management system
// Version 1.3:20120719
// ============================================================================
//
// Copyright (c) 2010-2012 General Development
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// ============================================================================
// 
// requires at least PHP verion 4.3 (for file_get_contents)
// Tested with PHP 5.2.10.



// Parameters:
// $_GET['l1'], $_GET['l2'],...etc. The sections of each level of the menu heirarchy tree to descend into until a file (not directory) is encountered.

//The following global variables assume default values unless set by the user.
if(!isset($use_url_rewriting)) $use_url_rewriting = FALSE;
if(!isset($href_prefix)) $href_prefix = '';
if(!isset($content_path)) $content_path = dirname(__FILE__) . '/content/';
if(!isset($gdcms_double_encode)) $gdcms_double_encode = FALSE;


//populate an array of sections for each heirarchy level until we get down to a leaf node.
// (files are leaf nodes, directories are not)
$_gdcms_last_level_specified = 0;
for($level=1; ; $level++)
{
    if(isset($targettitle)) unset($targettitle);
    if(isset($candidatefile)) unset($candidatefile);
    if(isset($_GET['l'.$level])) {
        $_gdcms_last_level_specified = $level;
        $targettitle = stripslashes($_GET['l'.$level]);
    }

    $dirlisting[$level] = scandir($content_path);

    foreach($dirlisting[$level] as $filename) {
        //parse the filename into index, title, etc.
        $delimpos = strpos($filename, '.');
        $designator = substr($filename, 0, $delimpos);
        if(!$designator) continue; //exclude 'files' like '..'
        $title = substr($filename, $delimpos+1);
        if(!is_dir($content_path . $filename)) { //non-directories have a filename extension (exclude it)
            $delimpos = strrpos($title, '.');
            if($delimpos !== FALSE)
                $title = substr($title, 0, $delimpos);
        }

        $index = 0;
        while(strlen($designator) > 0 && ctype_digit($temp = substr($designator, 0,1))) {
            $index .= $temp;
            $designator = substr($designator, 1);
        }
        $index = intval($index);
        if(strstr($designator, 'h') != FALSE) $hidden = TRUE; else $hidden = FALSE;
        if(strstr($designator, 'd') != FALSE) $default = TRUE; else $default = FALSE;
        if(strstr($designator, 'r') != FALSE) $redirect = TRUE; else $redirect = FALSE;


        //if it has a valid index, add the element to menu_titles...
        if($index > 0 && !$hidden) {
            $_gdcms_menu_titles[$level][$index] = $title;
        }

        
        //try to match it to targettitle, regardless of whether it has a menu index...
        $match = FALSE;
        if(isset($targettitle)) {
            if(strcasecmp($title, $targettitle)==0) {
                $match = TRUE;
            }
        } else if($default) {
            $match = TRUE;
        } else if($index == 1) {
            $match = TRUE;
        }
        if($match) {
            $_gdcms_level_index[$level] = $index;
            $candidatefile = $filename;
            $_gdcms_level_title[$level] = $title;
            $candidateredirect = $redirect;
        }
        
    }


    if(isset($candidatefile)) {
        $content_path .= $candidatefile;
        if(is_dir($content_path))
            $content_path .= '/'; //add trailing slash to directory
        else if($candidateredirect) {
            header('Location: ' . file_get_contents($content_path));
            exit();
        } else { //we have a leaf node!
            $_gdcms_http_status = 200; //OK (because content found)
            break;
        }
    } else {
        if($_gdcms_last_level_specified == $level) {
            $_gdcms_http_status = 404;
            $_gdcms_level_title[$level] = '[Not found]';
            header('HTTP/1.0 404 Not Found');
        } else {
            $_gdcms_http_status = 403;
            $_gdcms_level_title[$level] = '[Forbidden]';
            header('HTTP/1.1 403 Forbidden');
        }
        break;
    }
}
$depth = $level;


function gdcms_place_title($level=0) {
    global $_gdcms_level_title;
    global $depth;
    if(!$level) $level=$depth;
    if($level <= $depth) echo $_gdcms_level_title[$level];
}

function gdcms_place_content() {
    global $_gdcms_http_status;
    global $content_path;

    if($_gdcms_http_status == 200) //is it a leaf node?
        include($content_path);
    else if($_gdcms_http_status == 404)
        echo '<h1>Error<sub>404</sub></h1>The location wasn\'t found.';
    else
        echo '<h1>Error<sub>403</sub></h1>There is no content to browse at this level.';
}

function _gdcms_encode($url) {
    global $gdcms_double_encode;
    if($gdcms_double_encode)
        return urlencode(urlencode($url));
    return urlencode($url);
}

function gdcms_place_menu($level=0, $delim=' ') {
    global $_gdcms_menu_titles;
    global $_gdcms_level_index;
    global $_gdcms_level_title;
    global $_gdcms_last_level_specified;
    global $use_url_rewriting;
    global $href_prefix;
    global $depth;

    if(!$level) $level = $depth;

    if(isset($_gdcms_menu_titles[$level])) {
        $isfirstitem = TRUE;
        foreach ($_gdcms_menu_titles[$level] as $i => $title) { //print each link in order
            if(!$isfirstitem)
                echo $delim;
            $isfirstitem = FALSE;
            echo "<span id=\"menul{$level}s{$i}\" class=\"menuitem";
            $isactiveitem = ($i == $_gdcms_level_index[$level]);
            if($isactiveitem) echo ' menuactiveitem';
            echo "\">";
            if(!$isactiveitem || $level != $_gdcms_last_level_specified) {
                echo '<a href="' . $href_prefix;
                echo (!$use_url_rewriting) ? '?' : '/';
                for ($j=1; $j<$level; $j++) {
                    echo !$use_url_rewriting ? "l$j=" : '';
                    echo _gdcms_encode($_gdcms_level_title[$j]);
                    echo $use_url_rewriting ? '/' : '&';
                }
                echo !$use_url_rewriting ? "l$level=" : '';
                echo _gdcms_encode($title);
                echo '">';
                echo $title;
                echo '</a>';
            }
            else {
                echo $title;
            }
            echo '</span>';
        }
    }
}

function gdcms_place_promo() {
    echo '<a href="http://generaldevelopment.net/gdcms" title="The free, bloat-free, web content management system.">Powered by GD-CMS v1.</a>';
}

//deprecated functions to be removed in version 2...

function printtitle() { //deprecated function name.
    gdcms_place_title();
}

function printbody() { //deprecated function name.
    gdcms_place_content();
}

function printmenu($level) { //deprecated function name -- use gdcms_place_menu() instead.
    gdcms_place_menu($level);
}

?>
